@@ -115,6 +115,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     if (toll < 0)  stop("Negative tolerance ('tol' in seg.control()) is meaningless", call. = FALSE)
     visual <- control$visual
     stop.if.error <- fix.npsi <- control$fix.npsi
+    break.boot=control$break.boot
     n.boot <- control$n.boot
     size.boot <- control$size.boot
     gap <- control$gap
@@ -148,10 +149,11 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
-    if (class(mf$formula) == "name" && !"~" %in% paste(mf$formula)) 
+    if (class(mf$formula)[1] == "name" && !"~" %in% paste(mf$formula)) 
         mf$formula <- eval(mf$formula)
     #mf$formula <- update.formula(mf$formula, paste(seg.Z, collapse = ".+"))
     mf$formula <- update.formula1(mf$formula, paste(seg.Z, collapse = ".+"), opt=2)
+
     mfExt <- mf
     
     if (!is.null(obj$call$offset) || !is.null(obj$call$weights) || 
@@ -161,6 +163,9 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
                 all.vars(obj$call$subset), all.vars(obj$call$id)), 
                 collapse = "+")))
     }
+    
+    if(!is.null(obj$call$random)) mfExt$formula<-update.formula(mf$formula, paste(".~.+", paste(all.vars(obj$call$random), collapse="+")))
+    
     mf <- eval(mf, parent.frame())
     n <- nrow(mf)
     nomiOff <- setdiff(all.vars(formula(obj)), names(mf))
@@ -171,7 +176,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     nomiNO <- NULL
     for (i in nomiTUTTI) {
         r <- try(eval(parse(text = i), parent.frame()), silent = TRUE)
-        if (class(r) != "try-error" && length(r) == 1 && !is.function(r)) 
+        if (class(r)[1] != "try-error" && length(r) == 1 && !is.function(r) && !i%in%names(mf)) 
             nomiNO[[length(nomiNO) + 1]] <- i
         }
     if (!is.null(nomiNO)) 
@@ -332,7 +337,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     }
     else {
         obj <- seg.def.fit.boot(obj, Z, PSI, mfExt, opz, n.boot = n.boot, 
-            size.boot = size.boot, random = random)
+            size.boot = size.boot, random = random, break.boot=break.boot)
     }
     
     if (!is.list(obj)) {
@@ -390,7 +395,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     #all.coef <- coef(obj)
     #names(all.coef) <- c(names(coef(obj0)), nomiU, nomiVxb)
     #beta.c <- all.coef[nomiU]
-    beta.c<-coef(obj)[nomiU]
+    beta.c<-unlist( unique(coef(obj)[nomiU])) #beta.c<-coef(obj)[nomiU]
     Vxb <- V %*% diag(beta.c, ncol = length(beta.c))
     nnomi <- c(nomiU, nomiVxb)
     for (i in 1:ncol(U)) {
@@ -416,7 +421,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
     objF <- update(obj0, Fo, evaluate = FALSE, data = mfExt)
     if (!is.null(objF[["subset"]]))  objF[["subset"]] <- NULL
     if (is.null(opz$constr)) opz$constr <- 0
-    if ((opz$constr %in% 1:2) && class(obj0) == "rq") {
+    if ((opz$constr %in% 1:2) && class(obj0)[1] == "rq") {
         objF$method <- "fnc"
         objF$R <- quote(R)
         objF$r <- quote(r)
@@ -435,7 +440,7 @@ segmented.default<-function (obj, seg.Z, psi, npsi, fixed.psi=NULL, control = se
                 call. = FALSE)
             Fo <- update.formula1(formula(obj0), as.formula(paste(".~.+", 
                 paste(nomiU, collapse = "+"))), opt=1)
-            objF <- if ((opz$constr %in% 1:2) && class(obj0) == 
+            objF <- if ((opz$constr %in% 1:2) && class(obj0)[1] == 
                 "rq") {
                 update(obj0, formula = Fo, R = R.noV, r = r, 
                   method = "fnc", evaluate = TRUE, data = mfExt)
